
/*
    This code is based on Adafruit's work https://github.com/adafruit/Adafruit_ZeroI2S/blob/master/examples/tone_generator/tone_generator.ino
    
    Make an I2S Theremin using an Arduino MKRZero, an I2S DAC and two slider potentiometers!
*/
#include <SDU.h> 

#include <I2S.h>

/*const int maxFrequency = 5000; //maximum generated frequency
const int minFrequency = 220; //minimum generated frequency
const int maxVolume = 100; //max volume of the generated frequency
const int minVolume = 0; //min volume of the generated frequency

const int sampleRate = 44100; //samplerate of the generated frequency
const int wavSize = 256; //buffer size*/

#define SAMPLERATE_HZ 44100  // The sample rate of the audio.  Higher sample rates have better fidelity,
                             // but these tones are so simple it won't make a difference.  44.1khz is
                             // standard CD quality sound.

#define AMPLITUDE     5000   // Set the amplitude of generated waveforms.  This controls how loud
                             // the signals are, and can be any value from 0 to 65535.  Start with
                             // a low value like 5000 or less to prevent damaging speakers!

#define WAV_SIZE      256    // The size of each generated waveform.  The larger the size the higher
                             // quality the signal.  A size of 256 is more than enough for these simple
                             // waveforms.
							 
// Define the frequency of music notes (from http://www.phy.mtu.edu/~suits/notefreqs.html):
#define C4_HZ      261.63
#define D4_HZ      293.66
#define E4_HZ      329.63
#define F4_HZ      349.23
#define G4_HZ      392.00
#define A4_HZ      440.00
#define B4_HZ      493.88

// Define a C-major scale to play all the notes up and down.
float scale[] = { C4_HZ, D4_HZ, E4_HZ, F4_HZ, G4_HZ, A4_HZ, B4_HZ, A4_HZ, G4_HZ, F4_HZ, E4_HZ, D4_HZ, C4_HZ };

// Store basic waveforms in memory.
int16_t sine[WAV_SIZE]     = {0};
int16_t sawtooth[WAV_SIZE] = {0};
int16_t triangle[WAV_SIZE] = {0};
int16_t square[WAV_SIZE]   = {0};

const int frequencyPin = A0; //pin connected to the pot who determines the frequency of the signal
const int amplitudePin = A1; //pin connected to the pot who determines the amplitude of the signal

const int button = 6; //pin connected to the button to put the frequency as output

void setup() {
  Serial.begin(9600); //configue the serial ort
  Serial.println("Zero I2S Audio Tone Generator");

  // Initialize the I2S transmitter.
//  if (!I2S.begin(I2S_PHILIPS_MODE, sampleRate, 16)) {
  if (!I2S.begin(I2S_PHILIPS_MODE, SAMPLERATE_HZ, 16)) {
    Serial.println("Failed to initialize I2S!");
    while (1);
  }
//  generateSine(); // fill buffer with sine values
//  pinMode(button, INPUT_PULLUP); //put the button pin in input pullup

  // Generate waveforms.
  int amplitude = map(30, 0, 100, 0, 65535); //map amplitude
  Serial.print("Amplitude : ");
  Serial.println(amplitude,DEC);

  generateSine(amplitude, sine, WAV_SIZE);
  generateSawtooth(amplitude, sawtooth, WAV_SIZE);
  generateTriangle(amplitude, triangle, WAV_SIZE);
  generateSquare(amplitude, square, WAV_SIZE);

}

void loop() {
/*//  if (digitalRead(button) == LOW) {
//    float frequency = map(analogRead(frequencyPin), 0, 1023, minFrequency, maxFrequency); //map frequency
//    int amplitude = map(analogRead(amplitudePin), 0, 1023, minVolume, maxVolume); //map amplitude
    float frequency = 2000; //map frequency
    int amplitude = 80; //map amplitude
    playWave(frequency, 0.1, amplitude); //play sound
    //print values on serial
    Serial.print("Frequency = ");
    Serial.println(frequency);
    Serial.print("Amplitude = ");
    Serial.println(amplitude);
//  }*/

  Serial.println("Sine wave");
  for (int i=0; i<sizeof(scale)/sizeof(float); ++i) {
    // Play the note for a quarter of a second.
    playWave(sine, WAV_SIZE, scale[i], 0.25);
    // Pause for a tenth of a second between notes.
    delay(100);
  }
  Serial.println("Sawtooth wave");
  for (int i=0; i<sizeof(scale)/sizeof(float); ++i) {
    // Play the note for a quarter of a second.
    playWave(sawtooth, WAV_SIZE, scale[i], 0.25);
    // Pause for a tenth of a second between notes.
    delay(100);
  }
  Serial.println("Triangle wave");
  for (int i=0; i<sizeof(scale)/sizeof(float); ++i) {
    // Play the note for a quarter of a second.
    playWave(triangle, WAV_SIZE, scale[i], 0.25);
    // Pause for a tenth of a second between notes.
    delay(100);
  }
  Serial.println("Square wave");
  for (int i=0; i<sizeof(scale)/sizeof(float); ++i) {
    // Play the note for a quarter of a second.
    playWave(square, WAV_SIZE, scale[i], 0.25);
    // Pause for a tenth of a second between notes.
    delay(100);
  }
}

/*void generateSine() {
  for (int i = 0; i < wavSize; ++i) {
    sine[i] = ushort(float(100) * sin(2.0 * PI * (1.0 / wavSize) * i)); //100 is used to not have small numbers
  }
}*/

void generateSine(uint16_t amplitude, int16_t* buffer, uint16_t length) {
  // Generate a sine wave signal with the provided amplitude and store it in
  // the provided buffer of size length.
  for (int i=0; i<length; ++i) {
    buffer[i] = uint16_t(float(amplitude)*sin(2.0*PI*(1.0/length)*i));
  }
}
void generateSawtooth(uint16_t amplitude, int16_t* buffer, uint16_t length) {
  // Generate a sawtooth signal that goes from -amplitude/2 to amplitude/2
  // and store it in the provided buffer of size length.
  float delta = float(amplitude)/float(length);
  for (int i=0; i<length; ++i) {
    buffer[i] = -(amplitude/2)+delta*i;
  }
}

void generateTriangle(uint16_t amplitude, int16_t* buffer, uint16_t length) {
  // Generate a triangle wave signal with the provided amplitude and store it in
  // the provided buffer of size length.
  float delta = float(amplitude)/float(length);
  for (int i=0; i<length/2; ++i) {
    buffer[i] = -(amplitude/2)+delta*i;
  }
    for (int i=length/2; i<length; ++i) {
    buffer[i] = (amplitude/2)-delta*(i-length/2);
  }
}

void generateSquare(uint16_t amplitude, int16_t* buffer, uint16_t length) {
  // Generate a square wave signal with the provided amplitude and store it in
  // the provided buffer of size length.
  for (int i=0; i<length/2; ++i) {
    buffer[i] = -(amplitude/2);
  }
    for (int i=length/2; i<length; ++i) {
    buffer[i] = (amplitude/2);
  }
}


/*void playWave(float frequency, float seconds, int amplitude) {
  // Play back the provided waveform buffer for the specified
  // amount of seconds.
  // First calculate how many samples need to play back to run
  // for the desired amount of seconds.
  unsigned int iterations = seconds * sampleRate;
  // Then calculate the 'speed' at which we move through the wave
  // buffer based on the frequency of the tone being played.
  float delta = (frequency * wavSize) / float(sampleRate);
  // Now loop through all the samples and play them, calculating the
  // position within the wave buffer for each moment in time.
  for (unsigned int i = 0; i < iterations; ++i) {
    short pos = (unsigned int)(i * delta) % wavSize;
    short sample = amplitude * sine[pos];
    // Duplicate the sample so it's sent to both the left and right channel.
    // It appears the order is right channel, left channel if you want to write
    // stereo sound.
    while (I2S.availableForWrite() < 2);

    I2S.write(sample);
    I2S.write(sample);
  }
}*/

void playWave(int16_t* buffer, uint16_t length, float frequency, float seconds) {
  // Play back the provided waveform buffer for the specified
  // amount of seconds.
  // First calculate how many samples need to play back to run
  // for the desired amount of seconds.
  uint32_t iterations = seconds*SAMPLERATE_HZ;
  // Then calculate the 'speed' at which we move through the wave
  // buffer based on the frequency of the tone being played.
  float delta = (frequency*length)/float(SAMPLERATE_HZ);
  // Now loop through all the samples and play them, calculating the
  // position within the wave buffer for each moment in time.
  for (uint32_t i=0; i<iterations; ++i) {
    uint16_t pos = uint32_t(i*delta) % length;
    int16_t sample = buffer[pos];
    // Duplicate the sample so it's sent to both the left and right channel.
    // It appears the order is right channel, left channel if you want to write
    // stereo sound.
    I2S.write(sample);
    I2S.write(sample);
  }
}

